;TP2005b.asm - Touch Plate design patterned after tp2005.
;  Hold down touch 5 cycles light on 1 minute every hour

;Identical wiring of in/out ports as tp586.asm

;Current draw at idle = approx 1.6 ma (@5.5 volts)

;Internal oscillator in ATTINY2313 is running micro-controller at 1MHz
;r19 is routing number

;December 15, 2005

;----------------------------------------------------

;       HOLDDOWN, LED FLASH, COMMANDS:

;1.  Light turns off 8 minutes later
;2.  Light turns off 15 minutes later
;3.  Light turns off 30 minutes later
;4.  Light turns off 45 minutes later
;5.  Light turns on for one hour OR within 8 seconds,
;       Dit=1 hour,  Dah=10 hours
;	    Holddown flash of 5 = 1 minute on each hour
;6.  Random, Lived In, on time of 4 hours every day.  Light is 
;      on 10% of time.  The start time is when you go into this mode.
;7.  Random, Lived In, on time of 8 hours every day.  Light is 
;      on 10% of time.  The start time is when you go into this mode.
;8.  Random, Lived In, on time of 4 hours every day.  Light is 
;      on 50% of time.  The start time is when you go into this mode.
;9.  Random, Lived In, on time of 8 hours every day.  Light is 
;      on 50% of time.  The start time is when you go into this mode.
;10.  Random, Lived In, all the time.  Light is on 10% of time.  
;11.  Random, Lived In, all the time.  Light is on 50% of time.
;12.  Light turned on for 4 hours steady, every day beginning at 
;       the time this mode is selected.
;13.  Light turned on for 8 hours steady, every day beginning at
;       the time this mode is selected.
;14.  Light turned on each day at time this mode is entered.  Light 
;        must manually be turned off.  Short touches can toggle light
;        on and off without cancelling the daily on time.  Cancelation
;        of this mode is accomplish by any new holddown touch.
;15.  Delay start of cycle sequence.  15 flashes = 1 hour delay, 
;        16 flashes = two hours delay, 17 flashes = three hours 
;        delay, etc.  The last cycle mode used (flash modes 6 
;        through 9) is used when this mode is started.

;A short touch of the touch plate will toggle the light circuit on and off.  Short touches erase any previous Holddown operations.

;-------------------------------------------------------


.include "tn2313def.inc"

rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt
rjmp	START1
rjmp	START1

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ldi		r16,126		;Set bits PD1-6 to ones for out, bit PD0 for input
out		DDRD,r16	;Execute data direction with PD0 as input rest as output

clr		r16
out		PortD,r16	;LED and relay coil is off when port pin is low

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16
out		PORTB,R16	;execute to set out ports low

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock

ldi		r16,251		;was 245 199 - new 251,010
out		OCR1AH,r16
ldi		r16,10		;correction 11/28/2005 (for 586 Washroom TP2005)
out		OCR1AL,r16	;loads the compare register A for 4 second timer
					;intervals (1,000,000 clock frequency)	

clr		r11			;CCD interrupt hour counter
clr		r9			;hour countdown count
clr		r18			;clear flags register R5
clr		r31			;clear high byte of z register pair
clr		r29			;clear high bytes for y register pair
ldi		r28,$6A		;put start of memory for y register - $6A
mov		r30,r28		;put start of memory for z register

rcall	ResetTimers	;clear r5, 8, 9, 24, 25, 27, clear timer flags

rcall	TRoff		;turn off relay
ldi		r19,1		;default start routing to 1 (normal operation)
ldi		r16,6		;put default routing value of 6 into SRAM $61 (last used location)
sts		$61,r16


bset	7			;enable global interrupts

;-----------------------------


MainLoop:

sbic	PIND,0		;skip if switch is down (pressed)
rjmp	MainLoop2

rcall	ButtonPush2

cpi		r20,0		
brne	MLhc		;branch if it is a holddown count

cpi		r21,0		;short touch counter
brne	MLst
rjmp	MainLoop	;false down count - return to main loop


MLst:

cpi		r19,14
breq	MLst1

ldi		r19,1		;set routing to 1
clr		r5			;clear on/off counter
rjmp	MainLoop


MLst1:

rjmp	MainLoop2

;----------------------------------


;START OF HOLDDOWN COUNT DECODING

MLhc:

rcall	ResetTimers

ldi		r19,1		;reset routing to 1


ML1:

cpi		r20,1
brne	ML2


ML1b:

ldi		r16,8
mov		r5,r16		;put 8 minutes into r5 for on time counter


MLa:

ldi		r19,1		;set r19 for routing to 1


MLaa:

rcall	TRon		;turn on relay
rjmp	MLexit2

;-----------------------------


ML2:

cpi		r20,2
brne	ML3

ldi		r16,15
mov		r5,r16		;put 15 minutes into r5 on time counter
rjmp	MLa

;--------------------------------

ML3:

cpi		r20,3
brne	ML4

ldi		r16,30
mov		r5,r16		;put 30 minutes into r5 on time counter
rjmp	MLa

;--------------------------------


ML4:

cpi		r20,4
brne	ML5

ldi		r16,45
mov		r5,r16		;put 45 minutes into r5 on time counter

rjmp	MLa

;----------------------------------


ML5:

cpi		r20,5
brne	ML6

rcall	HD5			;rcall HD5 variable time with default of one hour on

rjmp	MLExit2		;go to MLExit2

;---------------------------


ML6:				;four hours on - 10% on 

cpi		r20,6
brne	ML7
ldi		r19,6		;set routing to 6 in r19
sts		$61,r19		;store "last used" into $61 RAM area

sbr		r18,64		;set r18, bit 6(64) - 24 hour flag on
rjmp	MLExit

;--------------------


ML7:				;eight hours on - 10% on

cpi		r20,7
brne	ML8

ldi		r19,7		;set routing to 7 in r19
sts		$61,r19		;store "last used" into $61 RAM area

sbr		r18,64		;set r18, bit 6(64) - 24 flag
rjmp	MLExit		;jump to 8 hour on start load


;------------------------


ML8:				;four hours on - 50% on

cpi		r20,8
brne	ML9

ldi		r19,8		;set routing to 8 in r19
sts		$61,r19		;store "last used" into $61 RAM area

sbr		r18,64		;set r18, bit 6(64) - 24 flag on
rjmp	MLExit

;--------------------------


ML9:				;eight hours on - 50% on

cpi		r20,9
brne	ML10

ldi		r19,9		;set routing to 9 in r19
sts		$61,r19		;store "last used" into $61 RAM area

sbr		r18,64		;set r18, bit 6(64) - 24 hr flag
rjmp	MLExit

;-----------------------------


ML10:				;on all the time - 10%

cpi		r20,10
brne	ML11

ldi		r19,10		;set routing to 10 in r19
cbr		r18,8		;clear r18, bit 3(8) - 10% on time
rcall	TRon		;turn on relay
ldi		r16,8
mov		r5,r16		;put 8 minutes on time for start in r5
rjmp	MLExit

;----------------------------


ML11:				;on all the time - 50%

cpi		r20,11
brne	ML12

ldi		r19,11		;set routing to 11 in r19
sbr		r18,8		;set r18, bit 3(8) - 50% on time
rcall	TRon		;turn on relay
ldi		r16,8
mov		r5,r16		;put 8 minutes on time for start in r5
rjmp	MLExit

;----------------------------


ML12:				;on every day, four hours solid

cpi		r20,12
brne	ML13

ldi		r19,12		;put routing number to 12
rcall	TRon		;turn on relay
ldi		r16,16
mov		r7,r16		;put 4 hours into 15 minute segment timer, r7
sts		$61,r19		;save "last used" routing
rjmp	MLExit

;------------------------------


ML13:				;on every day, eight hours solid

cpi		r20,13
brne	ML14

ldi		r19,13		;put routing number to 13
rcall	TRon		;turn on relay
ldi		r16,32		;put 8 hours into 13 minute segment timer, r7
sts		$61,r19		;save "last used" routing
mov		r7,r16		
rjmp	MLExit

;-------------------------------


ML14:				;On every day at time this mode selected
					; cancelled by entering any Holddown touch section

cpi		r20,14
brne	ML15

ldi		r19,14		;set routing to 14
sbr		r18,64		;set 24 hour flag on, r18, bit 6(64)

rjmp	MLExit

;-------------------------


ML15:				;delayed on - 15=1 hr, 16=2 hrs, 17=3 hrs, etc.

subi	r20,14		;subtract 14 from value in r20 (= number of hours for delay)

clr		r8			;clear delay on counter r8 to zero
ldi		r16,4		;four 15 min. segments per hour


ML15a:

add		r8,r16		;add four to value in r8 (one hour)
dec		r20
brne	ML15a

ldi		r19,15		;set routing to delay on for cycling
rcall	TRoff		;off relay

;---------------------------


MLExit:

rcall	FlashFour
rcall	Delay40th



MLExit2:

rcall	FlashFour
rjmp	MainLoop

;---------------------------


MainLoop2:

inc		r10			;increment random number counter

cpi		r19,1		
brne	ML25

tst		r5			;decrement r5 for routing one if not zero
breq	MainLoopX

sbrs	r18,0		;skip if r18, bit 0(1) is set - minute flag
rjmp	MainLoop

cbr		r18,1		;clear minute flag - r18, bit 0(1)
dec		r5		

brne	MainLoopX
rcall	TRoff		;off relay


DOSEnd:

rjmp	MainLoop


ML25:				;decrement hour count in r9

cpi		r19,5		
brne	ML26

tst		r9			;test for zero
brne	ML25a
rjmp	MainLoop


ML25a:

sbrs	r18,4		;skip if one hour flag set - r18,4(16)
rjmp	MainLoop

cbr		r18,16		;clear one hour flag, r18,4(16)
dec		r9
breq	ML25b
rjmp	MainLoop


ML25b:

rcall	TROff
ldi		r19,1		;reroute to r19=1 (normal)
rjmp	MainLoop


ML26:

cpi		r19,6		;4 hour on time, 10% on, every 24 hours
brne	ML27

rjmp	EN0			;entry point for 4 hour, 10% on time

;-------------------

MainLoopX:

rjmp	MainLoop
;--------------------


ML27:

cpi		r19,7		;8 hour on time, 10% on, every 24 hours
brne	ML28

rjmp	EN3			;entry point for 8 hour time

;---------------------


ML28:

cpi		r19,8		;4 hours on time, 50% on, every 24 hours
brne	ML29

rjmp	EN0A		;entry point for 4 hour time

;-----------------------


ML29:

cpi		r19,9		;8 hours on time, 50% on, every 24 hours
brne	ML30

rjmp	EN0B		;entry point for 8 hour time

;---------------------


ML30:

cpi		r19,10		;cycling ALL the time, 10% on
brne	ML31

rjmp	EN2b		;entry point for on all the time

;------------------------


ML31:

cpi		r19,11		;cycling ALL the time, 50% on
brne	ML32

rjmp	EN2b		;entry point for on all the time

;--------------------------


ML32:

cpi		r19,12		;light on steady for 4 hours every day at time programmed
brne	ML33

rjmp	EN5

;-------------------------------


ML33:

cpi		r19,13		;light on steady for 8 hours every day at tme programmed
brne	ML34

rjmp	EN6

;-----------------------


ML34:				;Turn on light every 24 hours (manual turnoff is necessary)

cpi		r19,14
brne	ML35

sbrs	r18,6		;skip if r18, bit 6(64) is set - 24 hour flag
rjmp	MainLoop

cbr		r18,64		;clear r18, bit 6(64) - 24 hour flag
rcall	TRon		;toggle relay on
rjmp	MainLoop


;---------------------------

ML35:				;handling of added delay before cycling will start
					;14=1 hr delay, 15=2 hrs delay. 16=3 hrs delay, etc.
					;r8 contains 4X hr value

cpi		r19,15
brne	ML36

sbrs	r18,1		;skip if r18, bit 1(2) is set - 15 minute flag
rjmp	MainLoop

cbr		r18,2		;clear r18, bit 1(2), off 15 minute flag
dec		r8			;decrement delay counter which has number of 15 min. segments)

brne	ML36

lds		r19,$61		;get stored last used routing location

rcall	ResetTimers	;reset timers
sbr		r18,64		;set 24 hour flag to on, r18, bit 6(64)
rjmp	MainLoop	;start to execute new command


ML36:

cpi		r19,16
brne	ML37

rjmp	EN25		;enable endless one minute on every hour


ML37:

rjmp	MainLoop

;-----------------------------


EN0:			;Lived In first 4 hours every 24 hours, 10% on, every day

sbrs	r18,6		;skip if r18, bit 6(64) is set - 24 hr flag
rjmp	EN2


EN1:

cbr		r18,64		;clear bit 6(64) - 24 hour flag
cbr		r18,8		;clear r18, bit 3(8) - 0 = 10% on time
ldi		r16,16		;put 4 hours (16, 15 min. segment counts) into 4/8 hour counter
mov		r7,r16		;put in counter r7, 4/8 counter


EN1a:

ldi		r16,8
mov		r5,r16		;on time of eight minutes 
rcall	TRon		;on relay
rjmp	EN2


EN1b:

ldi		r16,25
mov		r5,r16		;on time of 25 minutes
rcall	TRon		;on relay


EN2:

tst		r7			;test r7 for zero count - 4/8 hr timer
brne	EN2a
rjmp	MainLoop


EN2a:

sbrs	r18,1		;skip if r18, bit 1(2) is set - 15 minute flag
rjmp	EN2b

cbr		r18,2		;clear r18, bit 1(2) - 15 minute flag
dec		r7			;4/8 hour timer
breq	EN2c
rjmp	EN2b


EN2c:

rcall	TRoff		;off relay
rjmp 	MainLoop


EN2b:

sbrs	r18,0		;skip if r18, bit 0(1) is set - one minute flag
rjmp	MainLoop

cbr		r18,1		;clear r18, bit 0(1)
sbrs	r18,3		;skip if r18, bit 3(8) is set - 10%/90% flag
rjmp	DecrementTimer10


DecrementTimer90:	;cycles lights on and off - 50% on, 50% off

sbrs	r18,2		;skip if r18, bit 2(8) is set - relay flag on
rjmp	DT90a

dec		r5			;decrement on counter
brne	DT90end

rcall	TRoff		;off relay
rcall	PlaybackRandom
andi	r16,31		;save ls bits (0-31 minutes)
mov		r5,r16		;put value in r5 new off counter count	
ldi		r16,27
add		r5,r16		;add 27 minutes minimum off time	

DT90end:

rjmp	MainLoop


DT90a:

dec		r5			;decrement off counter, r5
brne	DT90end

rcall	TRon		;on relay
rcall	PlaybackRandom
andi	r16,31		;save ls bits (0-31 minutes)
mov		r5,r16		;put value in r5, new on counter count
ldi		r16,32		;add minimum of 32 minutes on time
add		r5,r16		;add to r5, on counter
rjmp	MainLoop

;-----------------------------------


EN3:				;8 hours on, 10%, every day

sbrs	r18,6		;skip if r18, bit 6(64) is set - 24 hour flag
rjmp	EN2


EN4:

cbr		r18,64		;clear r18, bit 6(64) - 24 hour flag
ldi		r16,32		;put 8 hours into 4/8 hour timer
mov		r7,r16
cbr		r18,8		;clear r18, bit 3(8) - 10% on time

rjmp	EN1a		;to continue code

;----------------------------


EN5:

sbrc	R18,6		;skip it R18, bit 6(64) is clear - 24 hour flag
rjmp	EN5a

tst		r7			;15 minute counter
breq	EN5End

sbrs	r18,1		;skip if r18, bit 1(2) is set - 15 minute flag
rjmp	MainLoop

cbr		r18,2		;clear r18, bit 1(2) - 15 minute flag
dec		r7			;decrement 15 minute segment counter

brne	EN5End

rcall	TRoff		;off relay


EN5end:

rjmp	MainLoop


EN5a:

cbr		r18,64		;clear r18, bit 6(64) - 24 hour flag
ldi		r16,16		;put 4 hours into r7, 15 minute counter, r7
mov		r7,r16
rcall	TRon		;on relay
rjmp	MainLoop

;-----------------------------


EN6:

sbrc	R18,6		;skip it R18, bit 6(64) is clear - 24 hour flag
rjmp	EN6a

tst		r7			;15 minute counter
breq	EN6End

sbrs	r18,1		;skip if r18, bit 1(2) is set - 15 minute flag
rjmp	MainLoop

cbr		r18,2		;clear r18, bit 1(2) - 15 minute flag
dec		r7			;decrement 15 minute segment counter

brne	EN6End

rcall	TRoff		;off relay


EN6end:

rjmp	MainLoop


EN6a:

cbr		r18,64		;clear r18, bit 6(64) - 24 hour flag
ldi		r16,32		;put 8 hours into r7, 15 minute counter, r7
mov		r7,r16
rcall	TRon		;on relay
rjmp	MainLoop


;-----------------------------


EN0A:				;on every day, 4 hours cycling, 50% on

sbrs	r18,6		;skip if r18, bit 6(64) is set - 24 hr flag
rjmp	EN2


EN0A1:

cbr		r18,64		;clear bit 6(64) - 24 hour flag
cbr		r18,8		;clear r18, bit 3(8) - 0 = 10% on time
ldi		r16,16		;put 4 hours (16, 15 min. segment counts) into 4/8 hour counter
mov		r7,r16		;put in counter r7, 4/8 counter
rjmp	EN1b		;to finish code

;-------------------------------


EN0b:				;on every day, 8 hours cycling, 50% on

sbrs	r18,6		;skip if r18, bit 6(64) is set - 24 hr flag
rjmp	EN2


EN0b1:

cbr		r18,64		;clear bit 6(64) - 24 hour flag
sbr		r18,8		;set r18, bit 3(8) - 0 = 50% on time
ldi		r16,32		;put 8 hours (16, 15 min. segment counts) into 4/8 hour counter
mov		r7,r16		;put in counter r7, 4/8 counter
rjmp	EN1b		;to finish code


DecrementTimer10:	;cycles lights on 10%, off rest of time

sbrs	r18,2		;skip if r18, bit 2(8) is set - relay flag on
rjmp	DT10a

dec		r5			;decrement on counter
brne	DT10end

rcall	TRoff		;off relay
rcall	PlaybackRandom
andi	r16,31		;save ls bits (0-31 minutes)
mov		r5,r16		;put value in r5 new off counter count	
ldi		r16,27
add		r5,r16		;add 27 minutes minimum off time	

DT10end:

rjmp	MainLoop


DT10a:

dec		r5			;decrement off counter, r5
brne	DT10end

rcall	TRon		;on relay
rcall	PlaybackRandom
andi	r16,7		;save ls bits (0-7 minutes)
mov		r5,r16		;put value in r5, new on counter count
ldi		r16,2		;add minimum of two minutes on time
add		r5,r16		;add to r5, on counter
rjmp	MainLoop


EN25:				;endless one minute on - 59 minutes off cycle

sbrs	r18,0		;skip if one minute flag, r18,0(1) is set
rjmp	MainLoop

cbr		r18,1		;clear one min flag, r18,0(1)

sbrs	r18,2		;skip if relay on flag is set, r18,2(4)
rjmp	EN25a

dec		r5
brne	EN25b		;jump back to Main Loop

ldi		r16,59		;put 59 minutes into r5 for off time
mov		r5,r16

rcall	TRoff		;turn relay off


EN25b:

rjmp	MainLoop


EN25a:				;relay is off

dec		r5
brne	EN25b		;jump back into Main Loop

ldi		r16,1		;put one minute on time into counter
mov		r5,r16

rcall	TRon		;turn relay on

rjmp	MainLoop

;------------------------


ClockCntDwnInt: 	;interrupt to update clock counters

push	r16
push 	r31

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,32		;set 4 second flag, r18,5(32)
inc		r24			;increment r24 (one minute counter)
cpi		r24,15		; (four second interrupt)
breq	CCD1	
rjmp	CCD2


CCD1:

clr		r24			;zero one minute counter
sbr		r18,1		;set bit 0 (1) in R18, flags register - one minute flag


CCD2:

inc		r27			;increment 15 minute count register, r27
cpi		r27,225		;compare for 15 minute count
breq	CCD3
rjmp	CCDIExit


CCD3:

clr		r27			;zero 15 minute count register
sbr		r18,2		;set r18 bit 1(2) - 15 minute flag

inc		r11			;increment r11, one hour counter
mov		r16,r11		
cpi		r16,4
breq	CCD3a


CCD3b:

inc		r25			;24 hour counter

cpi		r25,96		
breq	CCd4
rjmp	CCDIExit


CCD3a:

clr		r11			;reset r11, one hour timer to zero
sbr		r18,16		;set r18 bit 4(16) one hour flag
rjmp	CCD3b		;to continue


CCD4:

clr		r25			;clear r25, 24 hour counter
sbr		r18,64		;set r18, bit 6(64) - 24 hour flag


CCDIExit:			;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r31
pop		r16

reti

;------------------------


Delay40th:			; 1/40th second delay

push	r31
push	r0
push	r1

inc		r10			;increment random number counter

ldi		r31,33		; high byte
mov		r1,r31		; set up r1 for 1/40th second delay
clr		r0
ldi		r31,1		;no delay for r31


DelayA:

dec		r0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret

;----------------------------------


DelayPoint5:		;1/2 second delay

push	r16
ldi		r16,20
rjmp	Delay6a

;------------------------------------


Delay6:				;6 second delay

push	r16
clr		r16


Delay6a:

rcall	Delay40th
dec		r16
brne	Delay6a

pop		r16
ret

;---------------------------------------


Delay15:			;15 second delay routine

push 	r0
push	r16
clr		r0
ldi		r16,3
mov		r0,r16
clr		r16


Delay15a:

rcall	Delay40th	;1/40th second delay
dec		r16
brne	Delay15a

dec		r0
brne	Delay15a

pop		r16
pop		r0
ret

;------------------------------------


ToggleRelay:

sbrs	r18,2		;skip if bit 2 (4) flag is set (relay is on)
rjmp	TRon


TRoff:					; OFF RELAY

sbi		PortB,7			;on PortB,7 bit 
rcall	Delay40th
cbi		PortB,7			;off PortB,7 bit (pulse relay to off)
cbr		r18,4			;clear r18,2(4) bit.  Relay is now off
ret


TRon:					; ON RELAY

sbi		PortB,6			;on PortB,6 bit 
rcall	Delay40th
cbi		PortB,6			;off PortB,6 bit (pulse relay to on)
sbr		r18,4			;set r18,2(4) bit.  Relay is now on
ret

;-------------------------------------------


RecordRandom:		;puts new random numbers in area between $6A & $73 (10 locations)

st		y+,r10		;put random value in r10 into store location and post increment

cpi		r28,$74		;compare y register low byte for end of memory
breq	RR1

ret


RR1:

ldi		r28,$6A		;set pointer to start of memory

ret


;------------------------------

PlaybackRandom:		;retrieves random number from memory location pointed by z reg.

ld		r16,z+		;load random number into r16 and post increment

cpi		r30,$74		;compare z register low byte for end of memory
breq	PR1
rjmp	PR2


PR1:

ldi		r30,$6A		;put start of memory into low byte of z register


PR2:

tst		r16
breq	PR3
ret


PR3:

mov		r16,r10		;get random number from r10
ret

;------------------------------------


FlashOnce:

push	r16
ldi		r16,1
rjmp	FF1

;---------------------------


FlashFour:			;rapid four flashes of LED

push r16

ldi	r16,4


FF1:

rcall	Delay40th
sbi		PortB,0		;on LED
rcall	Delay40th
cbi		PortB,0		;off LED
rcall	Delay40th

dec		r16
brne	FF1

pop		r16
ret

;--------------------------------------


ButtonPush2:	;toggles Relay and gives down count in r20

clr		r20			;clear down count
clr		r21			;clear short touch count

rcall	Delay40th

sbic	PIND,0		;skip if switch is clear (down), Port B, bit 7
ret

push	r1

inc		r21			;increment short touch count

rcall	ToggleRelay
ldi		r16,66
mov		r1,r16		;put 66 count into r1 (1 1/2 second delay)


BP2a:

rcall	Delay40th

sbic	PIND,0		;skip if switch is clear (down)
rjmp	BP2b

inc		r10			;increment random number counter
dec		r1			;r1= the flash counter, count
brne	BP2a

sbi		PortB,0		;on LED
rcall	Delay40th	
cbi		PortB,0		;flash LED
ldi		r16,30
mov		r1,r16		;put 30 count into R1 delay counter - less than second delay
inc		r20			;increment down count by one

rjmp	BP2a


BP2b:

rcall	Delay40th
sbis	PIND,0		;skip if switch is clear (down)
rjmp	BP2a


BP2exit:

rcall	RecordRandom	;record a new random number into memory
pop		r1

ret

;------------------------------------------------


ResetTimers:

bclr	7			;disable global interrupts to reset interrupt counters

clr		r24			;clear minute counter
clr		r27			;clear 15 minute counter
clr		r25			;clear 24 hour counter
clr		r5			;clear on/off counter
clr		r8			;clear delay on counter
clr		r9			;clear hour count
clr		r11			;clear CCD interrupt one hour, 15 min seg counter
cbr		r18,115		;clear 4 sec, 1 min, 15 min, 1 hour, 24 hr flags

bset	7			;enable global interrupts

ret

;------------------------------------------------------

;------------------------------------------------------


Test1:		;Test routine - flashes out value in r16

rcall	FlashOnce
rcall	DelayPoint5
dec		r16
brne	Test1

ret

;------------------------------------


DIMled:				;part of test routine - dim glow of LED

sbis	PIND,0		;skip if switch is up
rjmp	DIMled1

sbi		PortB,0		;on LED
nop
cbi		PortB,0		;off LED
nop
nop
nop
nop
nop
rjmp	DIMled


DIMled1:

rcall	DelayPoint5
ret


;---------------------------------------

Button:		;Replaces old ButtonPush.  1/12/2006

push	r23

clr		r20		;HoldDown count
clr		r21		;short touch count 1-9
clr		r26		;short touch count 1-xx (counts past nine)
clr		r23		;clear countdown register


B1:

inc		r23
rcall	Delay40th
cpi		r23,120		;delay before first holddown count - 40=one second - 3 seconds
brsh	B3			;branch if - it is a holddown count

sbis	PIND,0		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

ldi		r16,10
add		r21,r16

sbi		PortB,0
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th


B2c:

rcall	Delay40th
cbi		PortB,0

clr		r23			;clear r23


B2d:

sbis	PIND,0		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,120		;has switch been up 120 counts? 40=one second (3 seconds)
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

inc		r21			;step short count one count - a dit short touch command	
inc		r26			;increment ALL dit counter for counts past nine

sbi		PortB,0

rjmp	B2c


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,0
rcall	Delay40th
cbi		PortB,0		;flash PROGRAM LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,0		;skip if switch is down
rjmp	B4a

cpi		r23,30
brlo	B4

rjmp	B3


B4a:

rcall	Delay40th


B5:

pop		r23
ret


;-----------------------------------------

HD5:			;Hold down count of five processing for variable hours on time

rcall	TRon	;on relay
ldi		r16,3	;minimum sensing time of 8 seconds
mov		r5,r16


HD5b:

sbrs	r18,5  	;skip if r18,5(32) 4 second flag is set
rjmp	HD5d

cbr		r18,32	;clear r18,5(32) -  reset 4 second flag
dec		r5
breq	HD5e


HD5d:

sbic	PIND,0	;skip if touch plate is touched
rjmp	HD5b


HD5f:

rcall	Button
cpi		r20,0
brne	HD5g		;valid count in hold down count r20 register

mov		r9,r21		;move total hour dit count into r9
rjmp	HD5ee


HD5g:

cpi		r20,5
brne	HD5e


HD5h:

clr		r5
inc		r5			;put one minute into r5 for endless one minute on every hour routine

ldi		r19,16		;route to one minute on every hour function

ret


HD5e:

clr		r9
inc		r9			;put one hour count into r9 hour counter


HD5ee:

ldi		r19,5		;re-route to r19 mode 5 - hour countdown

ret

;---------------------------------------